package org.processmining.streamconformance.local.plugin;

import java.util.LinkedList;
import java.util.List;

import javax.swing.JPanel;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.streamconformance.local.model.LocalModelStructure;
import org.processmining.streamconformance.regions.gui.panels.NetworkConfiguration;

@Plugin(
	name = "Online Conformance - Local behavior",
	returnLabels = { "A configuration for local online conformance checker" },
	returnTypes = { LocalOnlineConformanceConfiguration.class },
	parameterLabels = {
		"Model", "Marking"
	},
	categories = PluginCategory.Analytics,
	help = "This plugin computes the conformance of a given model with respect to an event streams.",
	userAccessible = true)
public class LocalOnlineConformancePlugin {

	@PluginVariant(requiredParameterLabels = { 0, 1 })
	@UITopiaVariant(
		author = "A. Burattin",
		email = "",
		affiliation = "DTU")
	public LocalOnlineConformanceConfiguration plugin(UIPluginContext context, Petrinet net, Marking initMarking) throws Exception {
		// prepare the configuration panels
		NetworkConfiguration configNetwork = new NetworkConfiguration();
		
		List<Pair<String, JPanel>> configurations = new LinkedList<Pair<String, JPanel>>();
		configurations.add(new Pair<String, JPanel>("Network Setup", configNetwork));
		
		// ask the user for the configuration parameters
		InteractionResult result = InteractionResult.NEXT;
		int currentStep = 0;
		int nofSteps = configurations.size();
		boolean configurationOngoing = true;
		while (configurationOngoing && currentStep < nofSteps) {
			Pair<String, JPanel> config = configurations.get(currentStep);
			result = context.showWizard(
					config.getFirst(),
					currentStep == 0,
					currentStep == nofSteps - 1,
					config.getSecond());

			switch (result) {
			case NEXT:
				currentStep++;
				break;
			case PREV:
				currentStep--;
				break;
			case FINISHED:
				configurationOngoing = false;
				break;
			case CANCEL:
				return null;
			default:
				configurationOngoing = false;
				break;
			}
		}
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		
		// wizard is over now, collect the configuration
		
		// create the local structure
		LocalOnlineConformanceConfiguration locc = new LocalOnlineConformanceConfiguration();
		locc.setAddress(configNetwork.getAddress());
		locc.setPort(configNetwork.getPort());
		locc.setLocalModelStructure(new LocalModelStructure(context, net, initMarking));
		locc.setNoMaxParallelInstances(100); // TODO: hardcoded for the moment
		
		return locc;
	}
}
