package org.processmining.streamconformance.local.plugin.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.border.Border;

import org.processmining.streamconformance.regions.utils.GUICustomUtils;
import org.processmining.streamconformance.regions.utils.UIColors;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class LocalConformanceListEntryRenderer<E> extends JPanel implements ListCellRenderer<E> {

	private static final long serialVersionUID = -1863880605178129502L;
	protected static SimpleDateFormat df = new SimpleDateFormat("yyyy-mm-dd hh:mm:ss");
	protected static NumberFormat nf = new DecimalFormat("#0.00");

	private Color highlightBackground;
	private Border highlightBorder;
	private Color normalBackground;
	private Border normalBorder;

	private JPanel colorIndicator;
	private JLabel textLeft;

	/**
	 * Default constructor
	 */
	public LocalConformanceListEntryRenderer() {
		this(Color.white, BorderFactory.createEmptyBorder(7, 5, 7, 5), UIColors.lightLightGray, BorderFactory.createEmptyBorder(7, 5, 7, 5));
	}

	/**
	 * Class constructor with some parameters to be set
	 *
	 * @param highlightBackground
	 * @param highlightBorder
	 * @param normalBorder
	 */
	public LocalConformanceListEntryRenderer(Color highlightBackground, Border highlightBorder, Color normalBackground, Border normalBorder) {

		this.highlightBackground = highlightBackground;
		this.highlightBorder = highlightBorder;
		this.normalBackground = normalBackground;
		this.normalBorder = normalBorder;

		this.colorIndicator = SlickerFactory.instance().createRoundedPanel(16, Color.red);
		this.textLeft = new JLabel();

		textLeft.setFont(getFont().deriveFont(Font.PLAIN));
		textLeft.setOpaque(false);

		setOpaque(true);
		setLayout(new BorderLayout(5, 0));
		add(colorIndicator, BorderLayout.WEST);
		add(textLeft, BorderLayout.CENTER);
	}

	@Override
	public Component getListCellRendererComponent(JList<? extends E> list, E value, int index, boolean isSelected, boolean cellHasFocus) {

		LocalConformanceListEntry entry = (LocalConformanceListEntry) value;

		colorIndicator.setBackground(GUICustomUtils.fromWeightToColor(entry.getColorCost()));
		textLeft.setText("<html><b>Case ID:</b> <em>" + entry.getCaseId().substring(0, Math.min(20, entry.getCaseId().length()))
				+ "</em>&nbsp;</b><br>"
				+ "<b>Conformance:</b> " + nf.format(entry.getScore().getConformance()) + "<br>"
				+ "<b>Completeness:</b> " + nf.format(entry.getScore().getCompleteness()) + "<br>"
				+ "<b>Confidence:</b> " + nf.format(entry.getScore().getConfidence()) + "</html>");
		
		if (list.isSelectedIndex(index)) {
			setBorder(highlightBorder);
			setBackground(highlightBackground);
		} else {
			setBorder(normalBorder);
			setBackground(normalBackground);
		}

		return this;
	}
}
