package org.processmining.streamconformance.local.tests;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Comparator;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.deckfour.xes.in.XesXmlGZIPParser;
import org.deckfour.xes.in.XesXmlParser;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.cli.CLIContext;
import org.processmining.contexts.cli.CLIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.boot.Boot;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Bootable;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.CommandLineArgumentList;
import org.processmining.framework.util.Pair;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.connections.transitionsystem.TransitionSystemConnection;
import org.processmining.models.graphbased.directed.DirectedGraphElementWeights;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.transitionsystem.AcceptStateSet;
import org.processmining.models.graphbased.directed.transitionsystem.CoverabilityGraph;
import org.processmining.models.graphbased.directed.transitionsystem.StartStateSet;
import org.processmining.models.graphbased.directed.transitionsystem.TransitionSystem;
import org.processmining.models.graphbased.directed.transitionsystem.TransitionSystemFactory;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.petrinet.behavioralanalysis.CGGenerator;
import org.processmining.plugins.pnml.importing.PnmlImportNet;
import org.processmining.plugins.tsml.Tsml;
import org.processmining.plugins.tsml.importing.TsmlImportUtils;
import org.processmining.streamconformance.local.model.LocalConformanceTracker;
import org.processmining.streamconformance.local.model.LocalModelStructure;
import org.processmining.streamconformance.local.model.OnlineConformanceScore;
import org.processmining.streamconformance.local.plugin.LocalOnlineConformanceConfiguration;
import org.processmining.streamconformance.utils.PetrinetHelper;
import org.processmining.streamconformance.utils.XLogHelper;

public class RealWorldTest {
	public static void saveTSML(PluginContext context, TransitionSystem ts, File file) throws IOException {
		StartStateSet starts = new StartStateSet();
		AcceptStateSet accepts = new AcceptStateSet();
		DirectedGraphElementWeights weights = new DirectedGraphElementWeights();

		ConnectionManager cm = context.getConnectionManager();
		try {
			TransitionSystemConnection tsc = cm.getFirstConnection(TransitionSystemConnection.class, context, ts);
			if (tsc.hasWeights()) {
				weights = tsc.getObjectWithRole(TransitionSystemConnection.WEIGHTS);
			}
			starts = tsc.getObjectWithRole(TransitionSystemConnection.STARTIDS);
			accepts = tsc.getObjectWithRole(TransitionSystemConnection.ACCEPTIDS);

		} catch (ConnectionCannotBeObtained e) {
			/**
			 * No connected transition weights found, no problem.
			 */
		}
		GraphLayoutConnection layout;
		try {
			layout = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, ts);
		} catch (ConnectionCannotBeObtained e) {
			layout = new GraphLayoutConnection(ts);
		}

		Tsml tsml = new Tsml().marshall(ts, starts, accepts, weights, layout);
		String text = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n" + tsml.exportElement(tsml);

		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)));
		bw.write(text);
		bw.close();
		System.out.println("saved");
	}
	
	public static CoverabilityGraph loadTSML(PluginContext context, File file) throws Exception {
		TsmlImportUtils utils = new TsmlImportUtils();
		Tsml tsml = utils.importTsmlFromStream(context, new FileInputStream(file), file.getAbsolutePath(), -1);
		CoverabilityGraph ts = TransitionSystemFactory.newCoverabilityGraph(tsml.getLabel() + " (imported from "
				+ file + ")");
		StartStateSet starts = new StartStateSet();
		AcceptStateSet accepts = new AcceptStateSet();
		DirectedGraphElementWeights weights = new DirectedGraphElementWeights();
		GraphLayoutConnection layout = new GraphLayoutConnection(ts);
		tsml.unmarshall(ts, starts, accepts, weights, layout);
		context.addConnection(new TransitionSystemConnection(ts, weights, starts, accepts));
		context.addConnection(layout);
		return ts;
	}
	
	public static Object[] getCachedStructures(PluginContext context, Petrinet net, Marking initMarking, String filename) throws Exception {
		
		String cachePath = System.getProperty("user.dir") + "\\tests\\testfiles\\real-logs\\cache\\" + filename + "-";
		
		// build coverability graphs
		CoverabilityGraph coverabilityGraph = null;
		File cg = new File(cachePath + "cg");
		if (cg.exists()) {
			coverabilityGraph = loadTSML(context, cg);
		} else {
			coverabilityGraph = context.tryToFindOrConstructFirstNamedObject(
					CoverabilityGraph.class,
					CGGenerator.class.getAnnotation(Plugin.class).name(),
					null,
					null,
					net,
					initMarking);
			saveTSML(context, coverabilityGraph, cg);
		}
		
		// build coverability graph of unfolded net
		CoverabilityGraph coverabilityGraphUnfolded = null;
		File cgun = new File(cachePath + "cg-un");
		if (cgun.exists()) {
			coverabilityGraphUnfolded = loadTSML(context, cgun);
		} else {
			Pair<Petrinet, Marking> unfoldedTotal = PetrinetHelper.unfold(context, net);
			coverabilityGraphUnfolded = context.tryToFindOrConstructFirstNamedObject(
					CoverabilityGraph.class,
					CGGenerator.class.getAnnotation(Plugin.class).name(),
					null,
					null,
					unfoldedTotal.getFirst(),
					unfoldedTotal.getSecond());
			saveTSML(context, coverabilityGraphUnfolded, cgun);
		}
		
		// build coverability graph of dual net
		CoverabilityGraph coverabilityGraphDualUnfolded = null;
		File cgrevun = new File(cachePath + "cg-rev-un");
		if (cgrevun.exists()) {
			coverabilityGraphDualUnfolded = loadTSML(context, cgrevun);
		} else {
			Pair<Petrinet, Marking> dualNet = PetrinetHelper.computeDual(context, net);
			Pair<Petrinet, Marking> unfoldedDualNet = PetrinetHelper.unfold(context, dualNet.getFirst());
			coverabilityGraphDualUnfolded = context.tryToFindOrConstructFirstNamedObject(
					CoverabilityGraph.class,
					CGGenerator.class.getAnnotation(Plugin.class).name(),
					null,
					null,
					unfoldedDualNet.getFirst(),
					unfoldedDualNet.getSecond());
			saveTSML(context, coverabilityGraphDualUnfolded, cgrevun);
		}
		
		return new Object[] {coverabilityGraph, coverabilityGraphUnfolded, coverabilityGraphDualUnfolded};
	}

	@Plugin(
			name = "local-test",
			parameterLabels = { "Model", "Marking", "filename" },
			returnLabels = { "" },
			returnTypes = { LocalOnlineConformanceConfiguration.class },
			userAccessible = true)
	@UITopiaVariant(
			author = "A. Burattin",
			affiliation = "",
			email = "")
	public static LocalOnlineConformanceConfiguration plugin(PluginContext context, Petrinet net, Marking initMarking, String filename) throws Exception {

		Object[] res = getCachedStructures(context, net, initMarking, filename);
		CoverabilityGraph coverabilityGraph = (CoverabilityGraph) res[0];
		CoverabilityGraph coverabilityGraphUnfolded = (CoverabilityGraph) res[1];
		CoverabilityGraph coverabilityGraphDualUnfolded = (CoverabilityGraph) res[2];
		
		// create the local structure
		LocalOnlineConformanceConfiguration locc = new LocalOnlineConformanceConfiguration();
		locc.setLocalModelStructure(new LocalModelStructure(coverabilityGraph, coverabilityGraphUnfolded, coverabilityGraphDualUnfolded));
		locc.setNoMaxParallelInstances(10000); // TODO: hardcoded for the moment
		return locc;
	}
	
	@Plugin(name = "CLI", parameterLabels = {}, returnLabels = {}, returnTypes = {}, userAccessible = false)
	@Bootable
	public Object main(CommandLineArgumentList commandlineArguments) throws Exception {
		CLIContext globalContext = new CLIContext();
		CLIPluginContext context = new CLIPluginContext(globalContext, "c");
		String modelName = "sepsis-v2.pnml";
		
		System.out.println("Importing model....");
		Petrinet netUnfolded = context.tryToFindOrConstructFirstNamedObject(
				Petrinet.class,
				PnmlImportNet.class.getAnnotation(Plugin.class).name(),
				null,
				null,
				System.getProperty("user.dir") + "\\tests\\testfiles\\real-logs\\" + modelName);
		InitialMarkingConnection imc = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, netUnfolded);
		
		LocalOnlineConformanceConfiguration locc = context.tryToFindOrConstructFirstNamedObject(
				LocalOnlineConformanceConfiguration.class, "local-test", null, null,
				imc.getObjectWithRole("Net"), imc.getObjectWithRole("Marking"), modelName);
		
		LocalModelStructure lms = locc.getLocalModelStructure();
		LocalConformanceTracker lct = new LocalConformanceTracker(lms, 100);
		System.out.println("Model build");
		
		System.out.println("Importing log...");
		XesXmlParser parser = new XesXmlGZIPParser();
		XLog log = parser.parse(new File(System.getProperty("user.dir") + "\\tests\\testfiles\\real-logs\\Sepsis Cases - Event Log.xes.gz")).get(0);
		class OU {
			public Date time;
			public String activity;
			public String caseid;
			public String toString() {
				return "(" + time + ", " + caseid + ", " + activity + ")";
			}
		}
		List<OU> stream = new LinkedList<OU>();
		for(XTrace t : log) {
			for(XEvent e : t) {
				OU ou = new OU();
				ou.activity = XLogHelper.getName(e);
				ou.caseid = XLogHelper.getName(t);
				ou.time = XLogHelper.getTimestamp(e);
				stream.add(ou);
			}
		}
		stream.sort(new Comparator<OU>() {
			public int compare(OU o1, OU o2) {
				return o1.time.compareTo(o2.time);
			}
		});
		
		double totalEvents = 0;
		for (OU ou : stream) {
			String caseId = ou.caseid;
			String newEventName = ou.activity;
			OnlineConformanceScore s = lct.replayEvent(caseId, newEventName);
			totalEvents++;
			System.out.println(totalEvents + "\t" + ou + "\t" + s.getConformance() + "\t" + s.getCompleteness() + "\t" + s.getConfidence());
//			if (totalEvents % 1000 == 0) {
//				break;
//				System.out.println("Processing... (" + totalEvents + " events)");
//				System.out.println(totalEvents + "\t" + s);
//			}
		}
		
		System.out.println("Total events processed: " + totalEvents);
		
		return null;
	}
	
	public static void main(String[] args) throws Exception {
		Boot.boot(RealWorldTest.class, CLIPluginContext.class, args);
	}
}
